/*
	Copyright 2008 Amaury GRAILLAT
	Mail: amoweb26@laposte.net
	Jabber: amoweb@im.apinc.org
	WebSite: amoweb.olympe-network.com

	This file is part of KeyBoard SpeedTest.

	KeyBoard SpeedTest is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	KeyBoard SpeedTest is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with KeyBoard SpeedTest.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "math.h"

#include <QtGui>

#include "speedtestDLG.h"

#include <iostream>
#include <string>

using namespace std;

extern bool playing;
extern bool editReadOnly;

#include "myThread.h"
extern MyThread thread; 

speedTest::speedTest(QWidget *parent)
{
	cout << __FILE__ << " : speedTest::speedTest(QWidget *parent)" << endl;
	
	//Init variables:
	timeCount     = 60;
	editReadOnly  = true;

	//Init Scores:
	correct        = 0;
	wrong          = 0;
	correctChar    = 0;
  
	betterCorrect  = 0;
	betterWrong    = 0;

	worstCorrect   = 0;
	worstWrong     = 0;
 
	averageCorrect = 0;
	averageWrong   = 0;

	totalDuration  = 0;
	totalCorrect   = 0;
	totalWrong     = 0;

	//cpm
	betterCorrectChar = 0;
	worstCorrectChar = 0;
	averageCorrectChar = 0;

	originalText = "";
	userOriginalText = "";

	setupUi(this); // this sets up GUI

	// Edits:
	connect( lineEdit,   SIGNAL( textChanged(const QString &) ), this, SLOT( checkWord(const QString &) ) );
	connect( lineEdit_2, SIGNAL( textEdited(const QString &) ), this, SLOT( word() ));

	// Radios buttons:
	connect( radioButton_Verbes, SIGNAL( clicked() ), this, SLOT( modeChanged() ) );
	connect( radioButton_Linux,  SIGNAL( clicked() ), this, SLOT( modeChanged() ) );
	connect( radioButton_GPL,     SIGNAL( clicked() ), this, SLOT( modeChanged() ) );
	connect( radioButton_Mots,   SIGNAL( clicked() ), this, SLOT( modeChanged() ) );
	connect( radioButton_Texte,  SIGNAL( clicked() ), this, SLOT( modeChanged() ) );

	connect( spinBox,  SIGNAL( valueChanged(int) ), &thread, SLOT ( defineTime(int) ) );
	connect( this,     SIGNAL( reinitTimer(int) ),  &thread, SLOT ( defineTime(int) ) );

	connect( &thread,  SIGNAL( signalGameTimeout() ), this, SLOT( gameTimeout() ) );
	connect( &thread,  SIGNAL( tic(int) ),            this, SLOT( tac(int) ) );

	thread.start();

	//Default mode:
	radioButton_Verbes -> setChecked(1);
	modeChanged();

	lcdNumber -> setSegmentStyle(QLCDNumber::Filled);

	//Centre the windows
	QDesktopWidget desktop;
	QRect desktop_area = desktop.screenGeometry();
	int x = desktop_area.width()/2 - width()/2;
	int y = desktop_area.height()/2 - height()/2;
	move(x,y);
}


/* Slot */ void speedTest::tac( int time )
{
    int duration = spinBox -> value();

    float correctPerMinFloat = (float)correct / (float)(duration-time) * 60;
//     float wrongPerMinFloat   = (float)wrong   / (float)(duration-time) * 60;

    float correctCharPerMinFloat  = (float)correctChar / (float)(duration-time) * 60;
    
    int correctPerMin     = correctPerMinFloat;
//     int wrongPerMin       = wrongPerMinFloat;
    long correctCharPerMin = correctCharPerMinFloat;
    

    if(correctPerMin > 0 && correctPerMin < 9000)
    {
      topLastScoreLabel -> setText("<html><body><center><b>Score : " + QString::number(correctPerMin, 10) + "mpm (" + QString::number(correctCharPerMin, 10) + " cpm) </b></center></body></html>");
      lastScoreLabel  -> setText("<html><body>Dernier score : " + QString::number(correctPerMin, 10) + "mpm (" + QString::number(correctCharPerMin, 10) + " cpm) </body></html>");

      //Shows Précision:
      float precisionFloat = 100 - ((float)wrong / (float)correct *100);
      long  precision = (long)precisionFloat;
    
      float precisionAverageFloat = 100 - ((float) averageWrong / (float)averageCorrect * 100);
      long precisionAverage = (long)precisionAverageFloat;
  
      pcentWrongLabel -> setText("<html><body>Précision : " + QString::number(precision, 10) + "%</body></html>");
    }

    progressBar -> setValue(correctPerMin*2);

    lcdNumber->display( time );
}

//Slot : Game's Time out:
/* Slot */ void speedTest::gameTimeout()
{
    int duration = spinBox -> value();

     float correctPerMinFloat = (float)correct / (float)(duration) * 60;
     float wrongPerMinFloat   = (float)wrong   / (float)(duration) * 60;
//      float wrongCharPerMinFloat   = (float)wrongChar   / (float)(duration) * 60;
     float correctCharPerMinFloat = (float)correctChar   / (float)(duration) * 60;
    
     int correctPerMin = correctPerMinFloat;
     int wrongPerMin   = wrongPerMinFloat;
     long correctCharPerMin = correctCharPerMinFloat;
//      long wrongCharPerMin   = wrongCharPerMinFloat; 

    //Better score:
    if(betterCorrect < correctPerMin )
    {
      betterCorrect = correctPerMin;
      betterWrong   = wrongPerMin;
      betterCorrectChar = correctCharPerMin;
    }

    //worst score:
    if( (worstCorrect > correctPerMin) || (worstCorrect == 0) )
    {
      worstCorrect = correctPerMin;
      worstWrong   = wrongPerMin;
      worstCorrectChar = correctCharPerMin;
    }

    if(averageCorrect > 0)
    {
      averageCorrect = (averageCorrect + correctPerMin) / 2;
      averageWrong   = (averageWrong + correctPerMin) / 2;

      averageCorrectChar = (averageCorrectChar + correctCharPerMin) / 2;
    }
    else
    {
      averageCorrect = correctPerMin;
      averageWrong   = wrongPerMin;

      averageCorrectChar = correctCharPerMin;
    }

    //Total:
    totalCorrect  = totalCorrect  + correct;
    totalWrong    = totalWrong    + wrong;
    totalDuration = totalDuration + duration;
  
    float minutes = (float) totalDuration / 60;

    topLastScoreLabel -> setText("<html><body><center><b>Score : " + QString::number(correctPerMin, 10) + " (" + QString::number(wrongPerMin, 10) + ") mpm </b></center></body></html>");
    lastScoreLabel  -> setText("<html><body>Dernier score : " + QString::number(correctPerMin, 10) + " (" + QString::number(wrongPerMin, 10) + ") mpm </body></html>");

    totalLabel -> setText("Total : " + QString::number(totalCorrect, 10) + " (" + QString::number(totalWrong, 10) + ") en " + QString::number((long)minutes, 10) + "m " + QString::number( (long) roundl( 60 * (minutes - (long)minutes ) ), 10) + "s");
   
    //Average Score:
    averageScoreLabel -> setText("Moyenne : " + QString::number(averageCorrect, 10) + " mpm (" + QString::number(averageCorrectChar, 10) + " cpm)" );

    //The Worst Score:
    worstScoreLabel -> setText("Le plus bas : " + QString::number(worstCorrect, 10) + " mpm (" + QString::number(worstCorrectChar, 10) + " cpm)" );
    
    //The Better Score:
    betterScoreLabel -> setText("Le plus haut : " + QString::number(betterCorrect, 10) + " mpm (" + QString::number(betterCorrectChar, 10) + " cpm)" );

    //Shows Précision:
    float precisionFloat = 100 - ((float)wrong / (float)correct *100);
    long  precision = (long)precisionFloat;
    float precisionAverageFloat = 100 - ((float)averageWrong / (float)averageCorrect * 100);
    long  precisionAverage = (long)precisionAverageFloat;
    pcentWrongLabel -> setText("<html><body>Précision : " + QString::number(precision, 10) + "% (Moyenne : " + QString::number(precisionAverage, 10) + "%)</body></html>");

    
}

//Check if there is a word:
/* Slot */ void speedTest::checkWord(const QString &)
{
	cout << __FILE__ << " : void speedTest::checkWord(const QString &)" << endl;

	//Add a space if there isn't:
	if(!lineEdit_2 -> text().endsWith(" ")) lineEdit_2 -> setText(lineEdit_2 -> text() + " ");

	//The test is starting:
	if(!playing)
	{
	  if( letter_case -> isChecked() )    caseSensitive = Qt::CaseSensitive;
	  else    caseSensitive = Qt::CaseInsensitive;

	  //Start the game:
	  if(lineEdit_2 -> text().length() > 1) playing = true;
	}

	lineEdit -> setReadOnly(editReadOnly);
	spinBox  -> setReadOnly(playing);

	//Fill the edit:
	if( lineEdit_2 -> text().length() < 100 )
	{
	    //Add a space if there isn't:
	    if(!lineEdit_2 -> text().endsWith(" ")) lineEdit_2 -> setText(lineEdit_2 -> text() + " ");

	    if(userText)
	    {
		if( randomWord ) lineEdit_2 -> setText( lineEdit_2 -> text() + mix( userOriginalText ) );
		else lineEdit_2 -> setText( lineEdit_2 -> text() + userOriginalText );
	    }
	    else
	    {
	      if( randomWord ) lineEdit_2 -> setText( mix( lineEdit_2 -> text() + originalText ) );
	      else lineEdit_2 -> setText( lineEdit_2 -> text() + originalText );
	    }
	}

    
	//User type a space:
	QString entry = lineEdit -> text();

	if( entry.contains(" ", caseSensitive) )
 	{
		//user wrote the right word:
		if( !entry.compare( word(), caseSensitive ) )
		{
		    correctChar = correctChar + entry.length();
		    correct++;
		}
		else wrong++;
		
		lineEdit -> clear();
		
		QString contant = lineEdit_2 -> text();
		if(!contant.endsWith(" ")) contant = contant + " ";
		contant = contant.right(contant.length()- contant.indexOf(" ") - 1);
		lineEdit_2 -> setText(contant);
	}

	word();
}

/* Slot */ void speedTest::modeChanged()
{
	cout << __FILE__ << " : modeChanged() : ";

	emit reinitTimer    ( spinBox -> value() );
	lcdNumber -> display( spinBox -> value() );

	correct=0;
	wrong=0;
	playing = false;
	editReadOnly = false;

	lineEdit -> setReadOnly(editReadOnly);
	spinBox  -> setReadOnly(playing);

	QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));

	lineEdit_2 -> clear();
	lineEdit   -> clear();

	if(radioButton_Verbes -> isChecked())
	{
		cout << "radioButton_Verbes" << endl;
		lineEdit_2 -> setReadOnly(1);

		originalText = "acheter avancer ajouter diviser jouer poser tracer lisser monter plaquer passer saluer éviter appeler avancer ajouter demander jouer poser tracer lier monter remuer passer saluer étudier cacher commencer compter cacher frapper foncer repenser louer libérer dépasser repasser surpasser étudier étaler briser balancer baisser flotter gommer penser payer limer démonter repasser tailler étudier écouter aimer chanter danser donner jouer avancer passer tricher laisser manger placer casser sauter avaler ";

		randomWord = true;
		userText = false;

		originalText = mix(originalText);
		lineEdit_2 -> setText(originalText);

	}
	else if(radioButton_Linux -> isChecked())
	{
		cout << "radioButton_Linux" << endl;
		lineEdit_2 -> setReadOnly(1);

		originalText = "sudo apt-get install apache2 cd ping su gzip mount mv netstat date ls kill fgrep pwd unmount dmesg dir grep rm dd cat kdesu mktemp svn vdir ln echo touch sh hostname pwd mkdir ";

		randomWord = true;
		userText = false;

		originalText = mix(originalText);
		lineEdit_2 -> setText(originalText);

	}
	else if(radioButton_GPL -> isChecked())
	{
		cout << "radioButton_GPL" << endl;
		lineEdit_2 -> setReadOnly(1);

		originalText = "La présente Licence s'applique à tout Programme (ou autre travail) où figure une note, placée par le détenteur des droits, stipulant que ledit Programme ou travail peut être distribué selon les termes de la présente Licence. Le terme Programme désigne aussi bien le Programme lui-même que tout travail qui en est dérivé selon la loi, c'est-à-dire tout ouvrage reproduisant le Programme ou une partie de celui-ci, à l'identique ou bien modifié, et/ou traduit dans une autre langue (la traduction est considérée comme une modification). Chaque personne concernée par la Licence Publique Générale sera désignée par le terme Vous. Les activités autres que copie, distribution et modification ne sont pas couvertes par la présente Licence et sortent de son cadre. Rien ne restreint l'utilisation du Programme et les données issues de celui-ci ne sont couvertes que si leur contenu constitue un travail basé sur le logiciel (indépendemment du fait d'avoir été réalisé en lançant le Programme). Tout dépend de ce que le Programme est censé produire. Vous pouvez copier et distribuer des copies conformes du code source du Programme, tel que Vous l'avez reçu, sur n'importe quel support, à condition de placer sur chaque copie un copyright approprié et une restriction de garantie, de ne pas modifier ou omettre toutes les stipulations se référant à la présente Licence et à la limitation de garantie, et de fournir avec toute copie du Programme un exemplaire de la Licence. Vous pouvez demander une rétribution financière pour la réalisation de la copie et demeurez libre de proposer une garantie assurée par vos soins, moyennant finances. Vous pouvez modifier votre copie ou vos copies du Programme ou partie de celui-ci, ou d'un travail basé sur ce Programme, et copier et distribuer ces modifications selon les termes de l'article 1, à condition de Vous conformer également aux conditions suivantes : - Ajouter aux fichiers modifiés l'indication très claire des modifications effectuées, ainsi que la date de chaque changement. - Distribuer sous les termes de la Licence Publique Générale l'ensemble de toute réalisation contenant tout ou partie du Programme, avec ou sans modifications. - Si le Programme modifié lit des commandes de manière interactive lors de son exécution, faire en sorte qu'il affiche, lors d'une invocation ordinaire, le copyright approprié en indiquant clairement la limitation de garantie (ou la garantie que Vous Vous engagez à fournir Vous-même), qu'il stipule que tout utilisateur peut librement redistribuer le Programme selon les conditions de la Licence Publique Générale GNU, et qu'il montre à tout utilisateur comment lire une copie de celle-ci (exception : si le Programme original est interactif mais n'affiche pas un tel message en temps normal, tout travail dérivé de ce Programme ne sera pas non plus contraint de l'afficher). ";
		
		randomWord = false;
		userText = false;
		lineEdit_2 -> setText(originalText);
	}
	else if(radioButton_Mots -> isChecked())
	{
		userText = true;
		randomWord = true;
		lineEdit_2 -> setReadOnly(0);
		lineEdit_2 -> setText(userOriginalText); //Setted in checkWord at the begin of the test
		cout << "radioButton_Mots" << endl;

	}
	else if(radioButton_Texte -> isChecked())
	{
		userText = true;
		randomWord = false;
		lineEdit_2 -> setReadOnly(0);
		lineEdit_2 -> setText(userOriginalText); //Setted in checkWord at the begin of the test
		cout << "radioButton_Texte" << endl;
	}

	//Updates words:
	word();
}

//Update screen
// delete user word
/* slot */ QString speedTest::word( )
{
	cout << __FILE__ << " : QString speedTest::word()" << endl;

	QString contant = lineEdit_2 -> text();
 	if(playing == false)
	{
	  //It's an user entered text:
	  if(userText == true) userOriginalText = contant;

	  //Text to fill the edit
	  originalText = contant; 
	}

	//Deletes doubles spaces
	QString contantTemps = contant;
	while(true)
	{
	  contantTemps = contant;
	  contant = contant.replace("   ", " ");
	  contant = contant.replace("  ", " ");
	  if(contantTemps == contant) break;
	}

	lineEdit_2 -> setText(contant);

	QString wantedWord;
	if(contant.indexOf(" ") != -1)
	{
		QString stringDisplay;

		//Displays the first word:
		wantedWord = contant; //Vouloir un mot c'est être contant! ... ok je -----> []
		contant.remove(0, wantedWord.indexOf(" ")+1);
		wantedWord.remove(wantedWord.indexOf(" ")+1, wantedWord.length());
		
		stringDisplay = "<html><big><font color=\"red\">" + wantedWord + "</font></big>";

		//Displays words:
		QString nextWord;
		int newWord=0;
		while(newWord < 20 && stringDisplay.length() < 100)
		{
		  nextWord = contant;
		  contant.remove(0, nextWord.indexOf(" ")+1);
		  nextWord.remove(nextWord.indexOf(" ")+1, nextWord.length());
		  newWord++;

		  stringDisplay = stringDisplay + nextWord;
		}

		stringDisplay = stringDisplay + "</html>";
		
		label1 -> setText(stringDisplay);
	}

	return wantedWord;
}

QString speedTest::mix(QString string)
{
    cout << __FILE__ << " : QString speedTest::mix(QString string)" << endl;

    if (!string.length()) return "";
    QString stringAfter;

    long countWords = string.count(" ", Qt::CaseInsensitive);

    //Contains true is the word is already used:
    bool *usedWord = new bool[countWords];

    //Init the list to false:
    long nbr = 0;
    while(nbr < countWords)
    {
      usedWord[nbr] = false;
      nbr++;
    }

    int posWord=0;
    while(posWord != -1)
    {
      int randomValue = (int)(rand() % countWords );

      if(usedWord[randomValue] == false) posWord = randomValue;
      else
      {
	  posWord = countWords;
	  while(usedWord[posWord] == true)
	  {
	    posWord--;
	    if(posWord == 0)
	    {
	      posWord=-1;
	      break;
	    }
	  }
      }

      if(posWord != -1)
      {
	    stringAfter = stringAfter + string.section(" ", posWord, posWord) + " ";
	    usedWord[posWord] = true;
      }
      else break;
    }
    
    delete usedWord;

    return stringAfter;
}

/* Slot */ void speedTest::about() 
{
}

//~speedTest
//{
//	delete timer;
//      delete thread;
// }


